/** =======================================================================
*** Net Image Filter -- a netpbm based image filtering service
*** 
*** by Marco Scheurer, Sen:te & Bill Bumgarner, Netsurfer, Inc.
*** 
*** Version 1.1
*** 
*** Copyright 1995, Sente Sarl. All rights reserved. Copyright 1995,
*** Netsurfer, Inc.. All Rights Reserved.
*** 
*** This notice may not be removed from this source code.
*** 
*** This object is included in the MiscKit by permission from the author(s)
*** and its use is governed by the MiscKit license, found in the file
*** "LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
*** for a list of all applicable permissions and restrictions.
*** =======================================================================
**/

#import <appkit/appkit.h>
#import "NetImageConverter.h"

void main(int argc, char **argv)
{
	id convertController;
    id listener;
	BOOL isDebug = NO;
	BOOL spewSTDERR = NO;

    /* Argument decoding must be done BEFORE NetImageConverter is
       instantiated. The call to NXRegisterDefaults EATS argv --
       argv[0] remains valid, but the rest is munged into la-la land
       in support of NXArgv.  BOO!
     */


	if (argc >= 2) {
		int argI;

		for(argI = 1; argI < argc; argI++) {
            /* Turn on debugging if "-debug" is present on the
               command line.
             */
			if(!strcmp(argv[argI], "-debug"))
				isDebug = YES;
			else if(!strcmp(argv[argI], "-stderr"))
				spewSTDERR = YES;
			else {
                /* Some day, this should:
                   
                   - allow the user to specify arguments to be passed
                   to pnmtotiff [controlling compression and such].
                   
                   - allow the user to specify files on the command
                   line;  Filter would then filter these to TIFF
                   images and either write them on the pasteboard
                   (hmmm... cool feature) or write them to a file
                   of the same name w/the ".tiff" extension.
                   
                   Right now, it just spews an error message and
                   exit()s because <bbum@friday.com> has no time.
                 */
				fprintf(stderr, "Unrecognized option: {%s}\n", argv[argI]);
				exit (1);
			}
		}
	}	

	convertController = [NetImageConverter sharedInstance:isDebug];
	listener = [Listener new];

	[convertController setSpewSTDERR: spewSTDERR];
	
    [listener setServicesDelegate:convertController];
    [listener checkInAs:"Net Image Filter"];
    [listener addPort];
    [Listener run];

	exit(0);
}
